@@ -42,7 +42,9 @@ const mbfl_encoding mbfl_encoding_ucs4 = {
 	"UCS-4",
 	(const char *(*)[])&mbfl_encoding_ucs4_aliases,
 	NULL,
-	MBFL_ENCTYPE_WCS4BE
+	MBFL_ENCTYPE_WCS4BE,
+	&vtbl_ucs4_wchar,
+	&vtbl_wchar_ucs4
 };
 
 const mbfl_encoding mbfl_encoding_ucs4be = {
@@ -51,7 +53,9 @@ const mbfl_encoding mbfl_encoding_ucs4be = {
 	"UCS-4BE",
 	NULL,
 	NULL,
-	MBFL_ENCTYPE_WCS4BE
+	MBFL_ENCTYPE_WCS4BE,
+	&vtbl_ucs4be_wchar,
+	&vtbl_wchar_ucs4be
 };
 
 const mbfl_encoding mbfl_encoding_ucs4le = {
@@ -60,7 +64,9 @@ const mbfl_encoding mbfl_encoding_ucs4le = {
 	"UCS-4LE",
 	NULL,
 	NULL,
-	MBFL_ENCTYPE_WCS4LE
+	MBFL_ENCTYPE_WCS4LE,
+	&vtbl_ucs4le_wchar,
+	&vtbl_wchar_ucs4le
 };
 
 const struct mbfl_convert_vtbl vtbl_ucs4_wchar = {
